sents = {
    "eng": {
        "word": "THIS IS A TEST",
        "arpabet": "DH IH S IH Z AH0 T EH1 S T",
        "ipa": "ð ɪ s ɪ z ə t ˈɛ s t",
        "disc": "D I s I z @ t 'E s t",
        "xsampa": 'D I s I z @ t "E s t',
    },
    "amh": {
        "word": "ይህ ፈተና ነው",
        "ipa": "j ɨ h f ə . t ə . n a n ə w",
        "xsampa": "j 1 h f @ . t @ . n a n @ w",
    },
    "ben": {
        "word": "এটা একটা পরীক্ষা",
        "ipa": "e . ʈ a æ k . ʈ a p o . r i k . kʰ a",
        "xsampa": "e . t` a { k . t` a p o . r i k . k_h a",
    },
    "deu": {
        "word": "dies ist ein Test",
        "disc": "d 'i s 'I s t 'W n t 'E s t",
        "xsampa": 'd "i: s "I s t "ai n t "E s t',
        "ipa": "d ˈiː s ˈɪ s t ˈai n t ˈɛ s t",
    },
    "nld": {
        "word": "dit is een test",
        "disc": "d 'I t 'I s 'e n t 'E s t",
        "xsampa": 'd "I t "I s "e: n t "E s t',
        "ipa": "d ˈɪ t ˈɪ s ˈeː n t ˈɛ s t",
    },
    "cmn": {
        "word": "这 是 一个 测试",
        "callhome": "Z e4 S %4 i2 g &4 c &4 S %4",
        "xsampa": "t`s` e_1_5 s` M_1_5 i_3_1 k @_1_5 ts_h @_1_5 s` M_1_5",
        "ipa": "ʈʂ e˥˩ ʂ ɯ˥˩ i˧˥ k ə˥˩ tsʰ ə˥˩ ʂ ɯ˥˩",
    },
    "spa": {
        "word": "esto es una prueba",
        "callhome": "e1 s t o e1 s u1 n a p r w e1 B a",
        "xsampa": '"e s t o "e s "u n a p 4 w "e B a',
        "ipa": "ˈe s t o ˈe s ˈu n a p ɾ w ˈe β a",
    },
    "arz": {
        "word": "هذا اختبار",
        "callhome": "h a d A A1 x t i b A2 r",
        "xsampa": 'h a d a: "a: x t i b %a: 4',
        "ipa": "h a d aː ˈaː x t i b ˌaː ɾ",
    },
}
